@@ -1,6 +1,7 @@
 # SPDX-License-Identifier: GPL-2.0
 
 import argparse
+import copy
 import json
 import math
 import os
@@ -99,6 +100,10 @@ snapshot_formatters = [
                   lambda snapshot, record, raw, fmt, rbargs:
                   recency_hist_str(snapshot, record, raw, fmt),
                   'last accessed time to total size of the regions histogram'),
+        Formatter('<heatmap>',
+                  lambda snapshot, record, raw, fmt, rbargs:
+                  heatmap_str(snapshot, record, raw, fmt),
+                  'heatmap of the snapshot'),
         ]
 
 region_formatters = [
@@ -150,7 +155,7 @@ def temperature_sz_hist_str(snapshot, record, raw, fmt):
 
     temperatures = sorted(hist.keys())
     min_temp, max_temp = temperatures[0], temperatures[-1]
-    interval = int((max_temp - min_temp) / 10)
+    interval = max(int((max_temp - min_temp) / 10), 1)
     max_temp = interval * math.ceil(max_temp / interval) + 1
 
     hist2 = []
@@ -174,7 +179,7 @@ def temperature_sz_hist_str(snapshot, record, raw, fmt):
             max_sz_str = len(sz_str)
         hist2.append([trange_str, sz_str, sz])
     max_dots = 20
-    sz_interval = int((max_sz - min_sz) / max_dots)
+    sz_interval = max(int((max_sz - min_sz) / max_dots), 1)
     lines = []
     for trange_str, sz_str, sz in hist2:
         trange_str = '%s%s' % (trange_str,
@@ -201,7 +206,7 @@ def recency_hist_str(snapshot, record, raw, fmt):
 
     last_used_times = sorted(hist.keys())
     min_lut, max_lut = last_used_times[0], last_used_times[-1]
-    interval = int((max_lut - min_lut) / 10)
+    interval = max(int((max_lut - min_lut) / 10), 1)
     max_lut = interval * math.ceil(max_lut / interval) + 1
 
     hist2 = []
@@ -227,7 +232,7 @@ def recency_hist_str(snapshot, record, raw, fmt):
             max_sz_str = len(sz_str)
         hist2.append([trange_str, sz_str, sz])
     max_dots = 20
-    sz_interval = int((max_sz - min_sz) / max_dots)
+    sz_interval = max(int((max_sz - min_sz) / max_dots), 1)
     lines = []
     for trange_str, sz_str, sz in hist2:
         trange_str = '%s%s' % (trange_str,
@@ -240,6 +245,111 @@ def recency_hist_str(snapshot, record, raw, fmt):
         lines.append('%s %s %s' % (trange_str, sz_str, bar))
     return '\n'.join(lines)
 
+def region_in(start, end, regions):
+    for region in regions:
+        if region.end <= start:
+            continue
+        if end <= region.start:
+            break
+        return True
+    return False
+
+def region_after(addr, regions):
+    for region in regions:
+        if region.start > addr:
+            return region
+    return None
+
+class HeatPixel:
+    start = None
+    end = None
+    temperature = None
+    is_void = None
+
+    def __init__(self, start, end, regions, temperature_weights):
+        self.start = start
+        self.end = end
+
+        self.temperature = 0
+        self.is_void = True
+        for region in regions:
+            if region.end <= start:
+                continue
+            if end <= region.start:
+                return
+            self.is_void = False
+            self.add_temperature(region, temperature_weights)
+
+    def add_temperature(self, region, weights):
+        start = self.start
+        end = self.end
+        # caller should ensure region intersects with self
+        if start <= region.start and region.end <= end:
+            # region is in the pixel
+            pass
+        elif region.start < start and end < region.end:
+            # pixel is in the region
+            region = copy.deepcopy(region)
+            region.start = start
+            region.end = end
+        elif region.end > start:
+            # region intersecting right part
+            # <region>
+            #    <pixel>
+            region = copy.deepcopy(region)
+            region.start = start
+        else:
+            # region intersecting left part
+            #    <region>
+            # <pixel>
+            region = copy.deepcopy(region)
+            region.end = end
+        self.temperature += temperature_of(region, weights)
+
+def heatmap_str(snapshot, record, raw, fmt):
+    total_sz = 0
+    for region in snapshot.regions:
+        total_sz += region.size()
+    map_length = fmt.snapshot_heatmap_width
+    sz_unit = total_sz / map_length
+
+    start = snapshot.regions[0].start
+    pixels = []
+    min_temperature = None
+    max_temperature = None
+    while start < snapshot.regions[-1].end:
+        end = start + sz_unit
+        pixels.append(HeatPixel(
+            start, end, snapshot.regions, fmt.temperature_weights))
+        if pixels[-1].is_void is True:
+            next_region = region_after(end, snapshot.regions)
+            if next_region is None:
+                break
+            start = next_region.start
+        else:
+            pixel = pixels[-1]
+            if min_temperature is None or pixel.temperature < min_temperature:
+                min_temperature = pixel.temperature
+            if max_temperature is None or pixel.temperature > max_temperature:
+                max_temperature = pixel.temperature
+            start = end
+
+    max_color_level = _damo_ascii_color.max_color_level()
+    temperature_unit = (max_temperature - min_temperature) / max_color_level
+    # single region?
+    if temperature_unit == 0:
+        temperature_unit = max_temperature / max_color_level
+    dots = []
+    for pixel in pixels:
+        if pixel.is_void is True:
+            dots.append('[...]')
+            continue
+        temp_level = int(
+                (pixel.temperature - min_temperature) / temperature_unit)
+        dots.append(_damo_ascii_color.colored(
+            '%d' % temp_level, fmt.snapshot_heatmap_colorset, temp_level))
+    return ''.join(dots)
+
 def rescale(val, orig_scale_minmax, new_scale_minmax, logscale=True):
     '''Return a value in new scale
 
@@ -651,6 +761,9 @@ class RecordsVisualizationFormat:
     format_snapshot_tail = None
     format_region = None
 
+    snapshot_heatmap_width = None
+    snapshot_heatmap_colorset = None
+
     region_box_values = None
     region_box_min_max_height = None
     region_box_min_max_length = None
@@ -675,6 +788,8 @@ class RecordsVisualizationFormat:
         self.format_snapshot_head = args.format_snapshot_head
         self.format_snapshot_tail = args.format_snapshot_tail
         self.format_region = args.format_region
+        self.snapshot_heatmap_width = args.snapshot_heatmap_width
+        self.snapshot_heatmap_colorset = args.snapshot_heatmap_colorset
         self.region_box_values = args.region_box_values
         self.region_box_min_max_height = args.region_box_min_max_height
         self.region_box_min_max_length = args.region_box_min_max_length
@@ -704,6 +819,8 @@ class RecordsVisualizationFormat:
                 'format_snapshot_head': self.format_snapshot_head,
                 'format_snapshot_tail': self.format_snapshot_tail,
                 'format_region': self.format_region,
+                'snapshot_heatmap_width': self.snapshot_heatmap_width,
+                'snapshot_heatmap_colorset': self.snapshot_heatmap_colorset,
                 'region_box_values': self.region_box_values,
                 'region_box_min_max_height': self.region_box_min_max_height,
                 'region_box_min_max_length': self.region_box_min_max_length,
@@ -727,6 +844,16 @@ class RecordsVisualizationFormat:
         self.format_snapshot_head = kvpairs['format_snapshot_head']
         self.format_snapshot_tail = kvpairs['format_snapshot_tail']
         self.format_region = kvpairs['format_region']
+        # snapshot_heatmap_width introduced from v2.5.6
+        if 'snapshot_heatmap_width' in kvpairs:
+            self.snapshot_heatmap_width = kvpairs['snapshot_heatmap_width']
+        else:
+            self.snapshot_heatmap_width = 80
+        # snapshot_heatmap_colorset introduced from v2.5.6
+        if 'snapshot_heatmap_colorset' in kvpairs:
+            self.snapshot_heatmap_width = kvpairs['snapshot_heatmap_width']
+        else:
+            self.snapshot_heatmap_width = 'gray'
         self.region_box_values = kvpairs['region_box_values']
         self.region_box_min_max_height = kvpairs['region_box_min_max_height']
         self.region_box_min_max_length = kvpairs['region_box_min_max_length']
@@ -898,6 +1025,13 @@ def add_fmt_args(parser, hide_help=False):
             default='total size: <total bytes>',
             help='output format to show at the end of each snapshot'
             if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--snapshot_heatmap_width', default=80, type=int,
+            help='width of snapshot heatmap')
+    parser.add_argument(
+            '--snapshot_heatmap_colorset', default='gray',
+            choices=_damo_ascii_color.colorsets.keys(),
+            help='snapshot heatmap colorset')
     parser.add_argument('--format_region', metavar='<template>',
                         default=default_region_format,
                         help='output format to show for each memory region'
